/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Kommunikationsparameter zur Kommunikation mit der Landesmeldestelle.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class LMSParameter {

	/** Hostname oder IP-Adresse des Rechners der LMS. */
	private String lmsHostRechner = "";

	/** Benutzername am Rechners der LMS. */
	private String lmsHostNutzer = "";

	/** Passwort am Rechners der LMS. */
	private String lmsHostPasswort = "";

	/**
	 * Eigene ID für die zu versendenden Meldungen. Vorgabe für das Element
	 * 'IDOrganisation' in der XML-Datei bei zu versendenden Meldungen.
	 */
	private String eigeneID = "";

	/**
	 * Pfad des Ausgangsverzeichnisses. Pfadname auf dem LMS-Rechner, in dem die
	 * zu versendenden Meldungen abgelegt werden sollen. Wenn der Wert nicht
	 * gesetzt ist, werden die Dateien ohne Verzeichniswechsel transferiert,
	 * d.h. in das Verzeichnis, in dem der FTP-Client nach der Anmeldung steht.
	 */
	private String ausgangsVerzeichnis = "";

	/**
	 * Pfad des Eingangsverzeichnisses Pfadname auf dem lokalen Rechner, in dem
	 * die Meldungen der LMS erwartet werden.
	 */
	private String eingangsVerzeichnis = "";

	/**
	 * Intervall Wiederholung der Sendung. Intervall für die automatische
	 * Wiederholung der Übertragung gültiger ausgehender Meldungen bis zum Ende
	 * der Gültigkeitsdauer.
	 */
	private long sendeWiederholung = 0;

	/**
	 * Zeitintervall zur Überprüfung der Kommunikation. Die Kommunikation
	 * zwischen LMS und VRZ wird mit diesem Zyklus überwacht. Der Wert gilt für
	 * die Überwachung beider Richtungen. Standardwert: 5 min
	 */
	private long testIntervallKommunikation = 5 * 60 * 1000;

	/**
	 * Laufende Nummer der Meldung der letzten an die LMS versendeten
	 * RDS-Meldung.
	 */
	private long laufendeNummer = 0;

	/**
	 * Akzeptierte Location Codes Liste von Location Codes, für die
	 * RDS-Meldungen entgegengenommen werden. Wenn die Liste leer ist, werden
	 * alle Location Codes akzeptiert.
	 */
	List<Integer> akzeptierteLocationCodes = new ArrayList<Integer>();

	/**
	 * Ignorierte Location Codes. Liste von Location Codes, für die
	 * RDS-Meldungen ignoriert werden. Wenn die Liste leer ist, werden alle
	 * Location Codes akzeptiert.
	 */
	List<Integer> ignorierteLocationCodes = new ArrayList<Integer>();

	/**
	 * Aktualisiert die Parameter auf der Basis eines von DAV empfangenen
	 * Datensatzes.
	 *
	 * @param data
	 *            DAV-Datensatz
	 */
	public synchronized void aktualisieren(final Data data) {
		if (data == null) {
			throw new RuntimeException("Keine Parameter verfügbar");
		}

		lmsHostRechner = data.getTextValue("LMSHostRechner").getText();
		lmsHostNutzer = data.getTextValue("LMSHostNutzer").getText();
		lmsHostPasswort = data.getTextValue("LMSHostPasswort").getText();
		eigeneID = data.getTextValue("EigeneID").getText();
		ausgangsVerzeichnis = data.getTextValue("Ausgangsverzeichnis").getText();
		eingangsVerzeichnis = data.getTextValue("Eingangsverzeichnis").getText();
		sendeWiederholung = data.getTimeValue("SendeWiederholung").getMillis();
		testIntervallKommunikation = data.getTimeValue("TestIntervallKommunikation").getMillis();
		laufendeNummer = data.getUnscaledValue("LaufendeNummer").longValue();

		Data.Array locarray = data.getArray("AkzeptierteLocationCodes");
		int anzahl = locarray.getLength();
		if (anzahl == 0) {
			akzeptierteLocationCodes.clear();
		} else {
			for (int i = 0; i < anzahl; i++) {
				final Integer loc = getLocationCode(locarray.getItem(i).asReferenceValue());
				if (loc != null) {
					akzeptierteLocationCodes.add(loc);
				}
			}
		}

		locarray = data.getArray("IgnorierteLocationCodes");
		anzahl = locarray.getLength();
		if (anzahl == 0) {
			akzeptierteLocationCodes.clear();
		} else {
			for (int i = 0; i < anzahl; i++) {
				final Integer loc = getLocationCode(locarray.getItem(i).asReferenceValue());
				if (loc != null) {
					ignorierteLocationCodes.add(loc);
				}
			}
		}
	}

	/**
	 * Gibt die Liste der akzeptierten Locationcodes zur&uuml;ck.
	 *
	 * @return Liste akzeptierter Locationcodes
	 */
	public List<Integer> getAkzeptierteLocationCodes() {
		return akzeptierteLocationCodes;
	}

	/**
	 * Gibt das parametrierte Ausgangsverzeichnis zur&uuml;ck.
	 *
	 * @return Ausgangsverzeichnis
	 */
	public String getAusgangsverzeichnis() {
		return ausgangsVerzeichnis;
	}

	/**
	 * Gibt die parametrierte eigene ID zur&uuml;ck.
	 *
	 * @return eigene ID
	 */
	public String getEigeneID() {
		return eigeneID;
	}

	/**
	 * Gibt das parametrierte Eingangsverzeichnis zur&uuml;ck.
	 *
	 * @return Eingangsverzeichnis
	 */
	public String getEingangsverzeichnis() {
		return eingangsVerzeichnis;
	}

	/**
	 * Gibt die Liste der zu ignorierenden Locationcodes zur&uuml;ck.
	 *
	 * @return Liste zu ignorierender Locationcodes
	 */
	public List<Integer> getIgnorierteLocationCodes() {
		return ignorierteLocationCodes;
	}

	/**
	 * Gibt die aktuelle laufende Nummer f&uuml;r die zu versendenen
	 * RDS-Meldungen zur&uuml;ck.
	 *
	 * @return aktuelle laufende Nummer
	 */
	public long getLaufendeNummer() {
		return laufendeNummer;
	}

	/**
	 * Gibt den parametrierten Nutzernamen am LMS-Host zur&uuml;ck.
	 *
	 * @return Nutzername
	 */
	public String getLMSHostNutzer() {
		return lmsHostNutzer;
	}

	/**
	 * Gibt das parametrierte Passwort am LMS-Host zur&uuml;ck.
	 *
	 * @return Passwort
	 */
	public String getLMSHostPasswort() {
		return lmsHostPasswort;
	}

	/**
	 * Gibt den parametrierten LMS-Host zur&uuml;ck.
	 *
	 * @return Hostname LMS-Host
	 */
	public String getLMSHostRechner() {
		return lmsHostRechner;
	}

	/**
	 * Bestimmt den Location-Code des referenzierten Objektes.
	 *
	 * @param rv
	 *            Referenz-Objekt
	 * @return Location-Code oder null, wenn der Location-Code nicht bestimmt
	 *         werden kann
	 */
	private Integer getLocationCode(final ReferenceValue rv) {
		final SystemObject objekt = rv.getSystemObject();
		final AttributeGroup atg = objekt.getDataModel().getAttributeGroup("atg.tmcLocationCode"); //$NON-NLS-1$
		final Data rd = objekt.getConfigurationData(atg);
		if (rd != null) {
			return new Integer(rd.getUnscaledValue("TmcLocationCode").intValue());
		}

		return null;
	}

	/**
	 * Gibt die parametrierten Sendewiederholungen zur&uuml;ck.
	 *
	 * @return Anzahl der Sendewiederholungen
	 */
	public long getSendeWiederholung() {
		return sendeWiederholung;
	}

	/**
	 * Gibt das parametrierte Testintervall f&uuml;r die
	 * Kommunikationsverbindung zur&uuml;ck.
	 *
	 * @return Testintervall
	 */
	public long getTestIntervallKommunikation() {
		return testIntervallKommunikation;
	}

	/**
	 * Setzt das Ausgangsverzeichnis.
	 *
	 * @param verzeichnis
	 *            Ausgangsverzeichnis
	 */
	public void setAusgangsverzeichnis(final String verzeichnis) {
		ausgangsVerzeichnis = verzeichnis;
	}

	/**
	 * Setzt die aktuelle laufende Nummer f&uuml;r die zu versendenen
	 * RDS-Meldungen.
	 *
	 * @param laufendeNummer
	 *            neue aktuelle laufende Nummer
	 */
	public void setLaufendeNummer(final long laufendeNummer) {
		this.laufendeNummer = laufendeNummer;
	}

	/**
	 * Setzt den Nutzernamen am LMS-Host.
	 *
	 * @param nutzer
	 *            Nutzername
	 */
	public void setLMSHostNutzer(final String nutzer) {
		lmsHostNutzer = nutzer;
	}

	/**
	 * Setzt das Passwort am LMS-Host.
	 *
	 * @param passwort
	 *            Passwort
	 *
	 */
	public void setLMSHostPasswort(final String passwort) {
		lmsHostPasswort = passwort;
	}

	/**
	 * Setzt den LMS-Host.
	 *
	 * @param rechner
	 *            LMS-Rechner
	 */
	public void setLMSHostRechner(final String rechner) {
		lmsHostRechner = rechner;
	}
}
